"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initializeBqplotFigure = exports.resizeRenderer = exports.createRenderer = exports.ScaleType = void 0;
const THREE = __importStar(require("three"));
var ScaleType;
(function (ScaleType) {
    ScaleType[ScaleType["SCALE_TYPE_LINEAR"] = 1] = "SCALE_TYPE_LINEAR";
    ScaleType[ScaleType["SCALE_TYPE_LOG"] = 2] = "SCALE_TYPE_LOG";
})(ScaleType = exports.ScaleType || (exports.ScaleType = {}));
function createRenderer(canvas, context, pixelRatio) {
    const renderer = new THREE.WebGLRenderer({
        canvas,
        context,
        antialias: true,
        alpha: true,
        premultipliedAlpha: true,
    });
    renderer.setPixelRatio(pixelRatio);
    renderer.setSize(100, 100);
    renderer.setClearAlpha(0);
    const camera = new THREE.OrthographicCamera(-1 / 2, 1 / 2, 1 / 2, -1 / 2, -10000, 10000);
    camera.position.z = 10;
    return { renderer, camera };
}
exports.createRenderer = createRenderer;
function resizeRenderer(renderer, width, height) {
    renderer.camera.left = 0;
    renderer.camera.right = width;
    renderer.camera.bottom = 0;
    renderer.camera.top = height;
    renderer.camera.updateProjectionMatrix();
    renderer.renderer.setSize(width, height);
}
exports.resizeRenderer = resizeRenderer;
function initializeBqplotFigure(figure) {
    if (!figure.needsWebGLContext) {
        figure.needsWebGLContext = true;
    }
    if (!figure.extras.webGLRenderer) {
        const ext = figure.extras;
        // Create ThreeJS Renderer
        ext.webGLRenderer = createRenderer(figure.webGLCanvas, figure.webGLContext, figure.model.get('pixel_ratio') || window.devicePixelRatio);
        ext.webGLMarks = [];
        // Actual render function
        ext.webGLRender = function () {
            const fig = this;
            const renderer = fig.extras.webGLRenderer.renderer;
            fig.extras.webGLRenderer.renderer.setPixelRatio(fig.model.get('pixel_ratio') || window.devicePixelRatio);
            resizeRenderer(fig.extras.webGLRenderer, fig.plotareaWidth, fig.plotareaHeight);
            // Clear renderer
            renderer.autoClear = false;
            renderer.autoClearColor = new THREE.Color(0x000000);
            renderer.clear();
            // Render WebGL marks
            const marks = fig.extras.webGLMarks;
            for (const mark of marks) {
                mark.renderGL();
            }
        }.bind(figure);
        // Render request functions
        ext.webGLUpdate = function () {
            const fig = this;
            fig.extras.webGLRender();
            fig.extras.webGLUpdateRequested = false;
        }.bind(figure);
        ext.webGLRequestRender = function () {
            const fig = this;
            if (!fig.extras.webGLUpdateRequested) {
                fig.extras.webGLUpdateRequested = true;
                requestAnimationFrame(fig.extras.webGLUpdate);
            }
        }.bind(figure);
        // Render hook: Called when the Figure needs a synchronous render
        figure.renderHooks.webGLRender = ext.webGLRender;
        // Relayout hook: Called when there is a figure relayout
        figure.relayoutHooks.webGLRelayoutHook = ext.webGLRequestRender;
        // Event handlers
        figure.listenTo(figure, 'margin_updated', ext.webGLRequestRender);
        figure.listenTo(figure.model, 'change:pixel_ratio', ext.webGLRequestRender);
    }
}
exports.initializeBqplotFigure = initializeBqplotFigure;
//# sourceMappingURL=utils.js.map